package com.sun.java.swing.plaf.windows;

import javax.swing.border.EmptyBorder;
import sun.swing.StringUIClientPropertyKey;
import javax.swing.JList;
import javax.swing.plaf.basic.BasicComboBoxRenderer;
import java.awt.KeyboardFocusManager;
import javax.swing.UIManager;
import javax.swing.border.Border;
import javax.swing.JTextField;
import javax.swing.plaf.basic.BasicComboBoxEditor;
import java.awt.event.KeyListener;
import javax.swing.plaf.basic.BasicComboPopup;
import java.awt.ComponentOrientation;
import java.beans.PropertyChangeEvent;
import javax.swing.ButtonModel;
import java.awt.event.MouseEvent;
import java.awt.event.MouseAdapter;
import javax.swing.JComboBox;
import javax.swing.JButton;
import javax.swing.ComboBoxEditor;
import javax.swing.plaf.basic.ComboPopup;
import java.awt.Insets;
import java.awt.Container;
import java.awt.LayoutManager;
import java.awt.Dimension;
import java.awt.Component;
import javax.swing.ListCellRenderer;
import javax.swing.JPanel;
import sun.swing.DefaultLookup;
import java.awt.Rectangle;
import java.awt.Graphics;
import javax.swing.plaf.ComponentUI;
import javax.swing.JComponent;
import java.beans.PropertyChangeListener;
import java.awt.event.MouseListener;
import javax.swing.plaf.basic.BasicComboBoxUI;

public class WindowsComboBoxUI extends BasicComboBoxUI
{
    private static final MouseListener rolloverListener;
    private boolean isRollover;
    private static final PropertyChangeListener componentOrientationListener;
    
    public WindowsComboBoxUI() {
        this.isRollover = false;
    }
    
    public static ComponentUI createUI(JComponent component) {
        return new WindowsComboBoxUI();
    }
    
    @Override
    public void installUI(JComponent c) {
        super.installUI(c);
        this.isRollover = false;
        this.comboBox.setRequestFocusEnabled(true);
        if (XPStyle.getXP() != null && this.arrowButton != null) {
            this.comboBox.addMouseListener(WindowsComboBoxUI.rolloverListener);
            this.arrowButton.addMouseListener(WindowsComboBoxUI.rolloverListener);
        }
    }
    
    @Override
    public void uninstallUI(JComponent c) {
        this.comboBox.removeMouseListener(WindowsComboBoxUI.rolloverListener);
        if (this.arrowButton != null) {
            this.arrowButton.removeMouseListener(WindowsComboBoxUI.rolloverListener);
        }
        super.uninstallUI(c);
    }
    
    @Override
    protected void installListeners() {
        super.installListeners();
        XPStyle xp = XPStyle.getXP();
        if (xp != null && xp.isSkinDefined(this.comboBox, TMSchema.Part.CP_DROPDOWNBUTTONRIGHT)) {
            this.comboBox.addPropertyChangeListener("componentOrientation", WindowsComboBoxUI.componentOrientationListener);
        }
    }
    
    @Override
    protected void uninstallListeners() {
        super.uninstallListeners();
        this.comboBox.removePropertyChangeListener("componentOrientation", WindowsComboBoxUI.componentOrientationListener);
    }
    
    @Override
    protected void configureEditor() {
        super.configureEditor();
        if (XPStyle.getXP() != null) {
            this.editor.addMouseListener(WindowsComboBoxUI.rolloverListener);
        }
    }
    
    @Override
    protected void unconfigureEditor() {
        super.unconfigureEditor();
        this.editor.removeMouseListener(WindowsComboBoxUI.rolloverListener);
    }
    
    @Override
    public void paint(Graphics g, JComponent c) {
        if (XPStyle.getXP() != null) {
            this.paintXPComboBoxBackground(g, c);
        }
        super.paint(g, c);
    }
    
    TMSchema.State getXPComboBoxState(JComponent component) {
        TMSchema.State state = TMSchema.State.NORMAL;
        if (!component.isEnabled()) {
            state = TMSchema.State.DISABLED;
        }
        else if (this.isPopupVisible(this.comboBox)) {
            state = TMSchema.State.PRESSED;
        }
        else if (this.isRollover) {
            state = TMSchema.State.HOT;
        }
        return state;
    }
    
    private void paintXPComboBoxBackground(Graphics graphics, JComponent component) {
        XPStyle xp = XPStyle.getXP();
        if (xp == null) {
            return;
        }
        TMSchema.State xpComboBoxState = this.getXPComboBoxState(component);
        XPStyle.Skin skin = null;
        if (!this.comboBox.isEditable() && xp.isSkinDefined(component, TMSchema.Part.CP_READONLY)) {
            skin = xp.getSkin(component, TMSchema.Part.CP_READONLY);
        }
        if (skin == null) {
            skin = xp.getSkin(component, TMSchema.Part.CP_COMBOBOX);
        }
        skin.paintSkin(graphics, 0, 0, component.getWidth(), component.getHeight(), xpComboBoxState);
        return;
    }
    
    @Override
    public void paintCurrentValue(Graphics graphics, Rectangle bounds, boolean hasFocus) {
        XPStyle xp = XPStyle.getXP();
        if (xp != null) {
            bounds.x += 2;
            bounds.y += 2;
            bounds.width -= 4;
            bounds.height -= 4;
        }
        else {
            ++bounds.x;
            ++bounds.y;
            bounds.width -= 2;
            bounds.height -= 2;
        }
        if (!this.comboBox.isEditable() && xp != null && xp.isSkinDefined(this.comboBox, TMSchema.Part.CP_READONLY)) {
            ListCellRenderer renderer = this.comboBox.getRenderer();
            Component c;
            if (hasFocus && !this.isPopupVisible(this.comboBox)) {
                c = renderer.getListCellRendererComponent(this.listBox, this.comboBox.getSelectedItem(), -1, true, false);
            }
            else {
                c = renderer.getListCellRendererComponent(this.listBox, this.comboBox.getSelectedItem(), -1, false, false);
            }
            c.setFont(this.comboBox.getFont());
            if (this.comboBox.isEnabled()) {
                c.setForeground(this.comboBox.getForeground());
                c.setBackground(this.comboBox.getBackground());
            }
            else {
                c.setForeground(DefaultLookup.getColor(this.comboBox, this, "ComboBox.disabledForeground", null));
                c.setBackground(DefaultLookup.getColor(this.comboBox, this, "ComboBox.disabledBackground", null));
            }
            boolean shouldValidate = false;
            if ((c instanceof JPanel)) {
                shouldValidate = true;
            }
            this.currentValuePane.paintComponent(graphics, c, this.comboBox, bounds.x, bounds.y, bounds.width, bounds.height, (boolean)shouldValidate);
        }
        else {
            super.paintCurrentValue(graphics, bounds, hasFocus);
        }
    }
    
    @Override
    public void paintCurrentValueBackground(Graphics g, Rectangle bounds, boolean hasFocus) {
        if (XPStyle.getXP() == null) {
            super.paintCurrentValueBackground(g, bounds, hasFocus);
        }
    }
    
    @Override
    public Dimension getMinimumSize(JComponent c) {
        Dimension minimumSize = super.getMinimumSize(c);
        if (XPStyle.getXP() != null) {
            minimumSize.width += 5;
        }
        else {
            minimumSize.width += 4;
        }
        minimumSize.height += 2;
        return minimumSize;
    }
    
    @Override
    protected LayoutManager createLayoutManager() {
        return new BasicComboBoxUI.ComboBoxLayoutManager() {
            @Override
            public void layoutContainer(Container parent) {
                super.layoutContainer(parent);
                if (XPStyle.getXP() != null && WindowsComboBoxUI.this.arrowButton != null) {
                    Dimension size = parent.getSize();
                    Insets access$700 = WindowsComboBoxUI.access$700(WindowsComboBoxUI.this);
                    int width = WindowsComboBoxUI.this.arrowButton.getPreferredSize().width;
                    WindowsComboBoxUI.this.arrowButton.setBounds(WindowsGraphicsUtils.isLeftToRight((Container)parent) ? (size.width - access$700.right - width) : access$700.left, access$700.top, width, size.height - access$700.top - access$700.bottom);
                }
            }
        };
    }
    
    @Override
    protected void installKeyboardActions() {
        super.installKeyboardActions();
    }
    
    @Override
    protected ComboPopup createPopup() {
        return super.createPopup();
    }
    
    @Override
    protected ComboBoxEditor createEditor() {
        return new WindowsComboBoxUI.WindowsComboBoxEditor();
    }
    
    @Override
    protected ListCellRenderer createRenderer() {
        XPStyle xp = XPStyle.getXP();
        if (xp != null && xp.isSkinDefined(this.comboBox, TMSchema.Part.CP_READONLY)) {
            return new WindowsComboBoxUI.WindowsComboBoxRenderer();
        }
        return super.createRenderer();
    }
    
    @Override
    protected JButton createArrowButton() {
        XPStyle xp = XPStyle.getXP();
        if (xp != null) {
            return new WindowsComboBoxUI.XPComboBoxButton(xp);
        }
        return super.createArrowButton();
    }
    
    static Insets access$700(WindowsComboBoxUI windowsComboBoxUI) {
        return windowsComboBoxUI.getInsets();
    }
    
    static {
        rolloverListener = new MouseAdapter() {
            private void handleRollover(MouseEvent mouseEvent, boolean rollover) {
                JComboBox<Object> comboBox = this.getComboBox(mouseEvent);
                WindowsComboBoxUI windowsComboBoxUI = this.getWindowsComboBoxUI(mouseEvent);
                if (comboBox == null || windowsComboBoxUI == null) {
                    return;
                }
                if (!comboBox.isEditable()) {
                    ButtonModel model = null;
                    if (windowsComboBoxUI.arrowButton != null) {
                        model = windowsComboBoxUI.arrowButton.getModel();
                    }
                    if (model != null) {
                        model.setRollover(rollover);
                    }
                }
                windowsComboBoxUI.isRollover = rollover;
                comboBox.repaint();
                return;
            }
            
            @Override
            public void mouseEntered(MouseEvent mouseEvent) {
                this.handleRollover(mouseEvent, true);
            }
            
            @Override
            public void mouseExited(MouseEvent mouseEvent) {
                this.handleRollover(mouseEvent, false);
            }
            
            private JComboBox getComboBox(MouseEvent mouseEvent) {
                Object source = mouseEvent.getSource();
                JComboBox<Object> access$300 = null;
                if ((source instanceof JComboBox)) {
                    access$300 = (JComboBox)source;
                }
                else if ((source instanceof WindowsComboBoxUI.XPComboBoxButton)) {
                    access$300 = ((WindowsComboBoxUI.XPComboBoxButton)source).getWindowsComboBoxUI().comboBox;
                }
                return access$300;
            }
            
            private WindowsComboBoxUI getWindowsComboBoxUI(MouseEvent mouseEvent) {
                JComboBox<Object> comboBox = this.getComboBox(mouseEvent);
                WindowsComboBoxUI windowsComboBoxUI = null;
                if (comboBox != null && comboBox.getUI() instanceof WindowsComboBoxUI) {
                    windowsComboBoxUI = (WindowsComboBoxUI)comboBox.getUI();
                    return windowsComboBoxUI;
                }
                return windowsComboBoxUI;
            }
        };
        componentOrientationListener = new PropertyChangeListener() {
            @Override
            public void propertyChange(PropertyChangeEvent propertyChangeEvent) {
                Object source;
                if ("componentOrientation" == propertyChangeEvent.getPropertyName() && (source = propertyChangeEvent.getSource()) instanceof JComboBox && ((JComboBox)source).getUI() instanceof WindowsComboBoxUI) {
                    JComboBox<Object> comboBox = (JComboBox)source;
                    WindowsComboBoxUI windowsComboBoxUI = (WindowsComboBoxUI)comboBox.getUI();
                    if ((windowsComboBoxUI.arrowButton instanceof WindowsComboBoxUI.XPComboBoxButton)) {
                        ((WindowsComboBoxUI.XPComboBoxButton)windowsComboBoxUI.arrowButton).setPart((comboBox.getComponentOrientation() == ComponentOrientation.RIGHT_TO_LEFT) ? TMSchema.Part.CP_DROPDOWNBUTTONLEFT : TMSchema.Part.CP_DROPDOWNBUTTONRIGHT);
                    }
                }
            }
        };
    }
    
    private class XPComboBoxButton extends XPStyle.GlyphButton
    {
        public XPComboBoxButton(XPStyle xpStyle) {
            super(null, xpStyle.isSkinDefined(WindowsComboBoxUI.this.comboBox, TMSchema.Part.CP_DROPDOWNBUTTONRIGHT) ? ((WindowsComboBoxUI.this.comboBox.getComponentOrientation() == ComponentOrientation.RIGHT_TO_LEFT) ? TMSchema.Part.CP_DROPDOWNBUTTONLEFT : TMSchema.Part.CP_DROPDOWNBUTTONRIGHT) : TMSchema.Part.CP_DROPDOWNBUTTON);
            this.setRequestFocusEnabled(false);
        }
        
        @Override
        protected TMSchema.State getState() {
            TMSchema.State state = super.getState();
            XPStyle xp = XPStyle.getXP();
            if (state != TMSchema.State.DISABLED && WindowsComboBoxUI.this.comboBox != null && !WindowsComboBoxUI.this.comboBox.isEditable() && xp != null && xp.isSkinDefined(WindowsComboBoxUI.this.comboBox, TMSchema.Part.CP_DROPDOWNBUTTONRIGHT)) {
                state = TMSchema.State.NORMAL;
                return state;
            }
            return state;
        }
        
        @Override
        public Dimension getPreferredSize() {
            return new Dimension(17, 21);
        }
        
        void setPart(TMSchema.Part part) {
            super.setPart(WindowsComboBoxUI.this.comboBox, part);
        }
        
        WindowsComboBoxUI getWindowsComboBoxUI() {
            return WindowsComboBoxUI.this;
        }
    }
    
    @Deprecated
    protected class WindowsComboPopup extends BasicComboPopup
    {
        public WindowsComboPopup(JComboBox combo) {
            super(combo);
        }
        
        @Override
        protected KeyListener createKeyListener() {
            return new WindowsComboBoxUI.WindowsComboPopup.InvocationKeyHandler();
        }
        
        protected class InvocationKeyHandler extends BasicComboPopup.InvocationKeyHandler
        {
        }
    }
    
    public static class WindowsComboBoxEditor extends BasicComboBoxEditor.UIResource
    {
        @Override
        protected JTextField createEditorComponent() {
            JTextField editorComponent = super.createEditorComponent();
            Border border = (Border)UIManager.get("ComboBox.editorBorder");
            if (border != null) {
                editorComponent.setBorder(border);
            }
            editorComponent.setOpaque(false);
            return editorComponent;
        }
        
        @Override
        public void setItem(Object item) {
            super.setItem(item);
            Component focusOwner = KeyboardFocusManager.getCurrentKeyboardFocusManager().getFocusOwner();
            if (focusOwner == this.editor || focusOwner == this.editor.getParent()) {
                this.editor.selectAll();
            }
        }
    }
    
    private static class WindowsComboBoxRenderer extends BasicComboBoxRenderer.UIResource
    {
        private static final Object BORDER_KEY;
        private static final Border NULL_BORDER;
        
        @Override
        public Component getListCellRendererComponent(JList list, Object value, int index, boolean isSelected, boolean cellHasFocus) {
            Component listCellRendererComponent = super.getListCellRendererComponent(list, value, index, isSelected, cellHasFocus);
            if ((listCellRendererComponent instanceof JComponent)) {
                JComponent component = (JComponent)listCellRendererComponent;
                if (index == -1 && isSelected) {
                    Border border = component.getBorder();
                    component.setBorder(new WindowsBorders.DashedBorder(list.getForeground()));
                    if (component.getClientProperty(WindowsComboBoxUI.WindowsComboBoxRenderer.BORDER_KEY) == null) {
                        component.putClientProperty(WindowsComboBoxUI.WindowsComboBoxRenderer.BORDER_KEY, (border == null) ? WindowsComboBoxUI.WindowsComboBoxRenderer.NULL_BORDER : border);
                    }
                }
                else if ((component.getBorder() instanceof WindowsBorders.DashedBorder)) {
                    Object clientProperty = component.getClientProperty(WindowsComboBoxUI.WindowsComboBoxRenderer.BORDER_KEY);
                    if ((clientProperty instanceof Border)) {
                        component.setBorder((clientProperty == WindowsComboBoxUI.WindowsComboBoxRenderer.NULL_BORDER) ? null : ((Border)clientProperty));
                    }
                    component.putClientProperty(WindowsComboBoxUI.WindowsComboBoxRenderer.BORDER_KEY, null);
                }
                if (index == -1) {
                    component.setOpaque(false);
                    component.setForeground(list.getForeground());
                }
                else {
                    component.setOpaque(true);
                }
            }
            return listCellRendererComponent;
        }
        
        static {
            BORDER_KEY = new StringUIClientPropertyKey("BORDER_KEY");
            NULL_BORDER = new EmptyBorder(0, 0, 0, 0);
        }
    }
}
